
#include "LEDEmbedded.hh"

#include "Switch.hh"
#include "umlrtcommsportrole.hh"
#include "umlrtmessage.hh"
#include "umlrtslot.hh"
#include "umlrttimerprotocol.hh"
#include <cstddef>
#include "umlrtcapsuleclass.hh"
#include "umlrtframeservice.hh"
class UMLRTRtsInterface;
struct UMLRTCommsPort;

Capsule_LEDEmbedded::Capsule_LEDEmbedded( const UMLRTCapsuleClass * cd, UMLRTSlot * st, const UMLRTCommsPort * * border, const UMLRTCommsPort * * internal, bool isStat )
: UMLRTCapsule( NULL, cd, st, border, internal, isStat )
, lED( borderPorts[borderport_lED] )
, switch( borderPorts[borderport_switch] )
, timing( internalPorts[internalport_timing] )
, currentState( SPECIAL_INTERNAL_STATE_UNVISITED )
{
    stateNames[top__ON] = "top__ON";
    stateNames[top__ON__LEDOFF] = "top__ON__LEDOFF";
    stateNames[top__OFF] = "top__OFF";
    stateNames[top__ON__LEDON] = "top__ON__LEDON";
    stateNames[SPECIAL_INTERNAL_STATE_UNVISITED] = "<uninitialized>";
    int i = 0;
    while( i < 1 )
        history[i++] = SPECIAL_INTERNAL_STATE_UNVISITED;
}








void Capsule_LEDEmbedded::bindPort( bool isBorder, int portId, int index )
{
    if( isBorder )
        switch( portId )
        {
        case borderport_lED:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_lED, index, true );
            break;
        case borderport_switch:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_switch, index, true );
            break;
        }
    else
        UMLRTFrameService::sendBoundUnbound( internalPorts, internalport_timing, index, true );
}

void Capsule_LEDEmbedded::unbindPort( bool isBorder, int portId, int index )
{
    if( isBorder )
        switch( portId )
        {
        case borderport_lED:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_lED, index, false );
            UMLRTFrameService::disconnectPort( borderPorts[borderport_lED], index );
            break;
        case borderport_switch:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_switch, index, false );
            UMLRTFrameService::disconnectPort( borderPorts[borderport_switch], index );
            break;
        }
    else
    {
        UMLRTFrameService::sendBoundUnbound( internalPorts, internalport_timing, index, false );
        UMLRTFrameService::disconnectPort( internalPorts[internalport_timing], index );
    }
}

void Capsule_LEDEmbedded::inject( const UMLRTMessage & message )
{
    msg = &message;
    switch( currentState )
    {
    case top__OFF:
        currentState = state_____top__OFF( &message );
        break;
    case top__ON__LEDOFF:
        currentState = state_____top__ON__LEDOFF( &message );
        break;
    case top__ON__LEDON:
        currentState = state_____top__ON__LEDON( &message );
        break;
    default:
        break;
    }
}

void Capsule_LEDEmbedded::initialize( const UMLRTMessage & message )
{
    msg = &message;
    currentState = top__OFF;
}

const char * Capsule_LEDEmbedded::getCurrentStateString() const
{
    return stateNames[currentState];
}





void Capsule_LEDEmbedded::save_history( Capsule_LEDEmbedded::State compositeState, Capsule_LEDEmbedded::State subState )
{
    history[compositeState] = subState;
}

bool Capsule_LEDEmbedded::check_history( Capsule_LEDEmbedded::State compositeState, Capsule_LEDEmbedded::State subState )
{
    return history[compositeState] == subState;
}

void Capsule_LEDEmbedded::actionchain_____top__ON__new_transition_4__ActionChain20( const UMLRTMessage * msg )
{
    save_history( top__ON, top__ON__LEDOFF );
}

void Capsule_LEDEmbedded::actionchain_____top__ON__new_transition_5__ActionChain21( const UMLRTMessage * msg )
{
    save_history( top__ON, top__ON__LEDON );
}

Capsule_LEDEmbedded::State Capsule_LEDEmbedded::junction_____top__ON__new_exitpoint_1( const UMLRTMessage * msg )
{
    return top__OFF;
}

Capsule_LEDEmbedded::State Capsule_LEDEmbedded::choice_____top__ON__deephistory( const UMLRTMessage * msg )
{
    if( check_history( top__ON, top__ON__LEDOFF ) )
        return top__ON__LEDOFF;
    else if( check_history( top__ON, top__ON__LEDON ) )
        return top__ON__LEDON;
    else if( check_history( top__ON, SPECIAL_INTERNAL_STATE_UNVISITED ) )
        return top__ON__LEDOFF;
    return currentState;
}

Capsule_LEDEmbedded::State Capsule_LEDEmbedded::state_____top__OFF( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    case port_switch:
        switch( msg->getSignalId() )
        {
        case Switch::signal_switchOn:
            return choice_____top__ON__deephistory( msg );
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}

Capsule_LEDEmbedded::State Capsule_LEDEmbedded::state_____top__ON__LEDOFF( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    case port_timing:
        switch( msg->getSignalId() )
        {
        case UMLRTTimerProtocol::signal_timeout:
            return top__ON__LEDON;
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    case port_switch:
        switch( msg->getSignalId() )
        {
        case Switch::signal_switchOff:
            actionchain_____top__ON__new_transition_4__ActionChain20( msg );
            return junction_____top__ON__new_exitpoint_1( msg );
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}

Capsule_LEDEmbedded::State Capsule_LEDEmbedded::state_____top__ON__LEDON( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    case port_lED:
        switch( msg->getSignalId() )
        {
        case UMLRTTimerProtocol::signal_timeout:
            return top__ON__LEDOFF;
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    case port_switch:
        switch( msg->getSignalId() )
        {
        case Switch::signal_switchOff:
            actionchain_____top__ON__new_transition_5__ActionChain21( msg );
            return junction_____top__ON__new_exitpoint_1( msg );
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}


static const UMLRTCommsPortRole portroles_border[] = 
{
    {
        Capsule_LEDEmbedded::port_lED,
        "LED",
        "lED",
        "",
        1,
        true,
        false,
        false,
        false,
        false,
        false,
        true
    },
    {
        Capsule_LEDEmbedded::port_switch,
        "Switch",
        "switch",
        "",
        1,
        true,
        false,
        false,
        false,
        false,
        false,
        true
    }
};

static const UMLRTCommsPortRole portroles_internal[] = 
{
    {
        Capsule_LEDEmbedded::port_timing,
        "Timing",
        "timing",
        "",
        1,
        true,
        false,
        false,
        false,
        false,
        false,
        true
    }
};

static void instantiate_LEDEmbedded( const UMLRTRtsInterface * rts, UMLRTSlot * slot, const UMLRTCommsPort * * borderPorts )
{
    const UMLRTCommsPort * * internalPorts = UMLRTFrameService::createInternalPorts( slot, &LEDEmbedded );
    slot->capsule = new Capsule_LEDEmbedded( &LEDEmbedded, slot, borderPorts, internalPorts, false );
}

const UMLRTCapsuleClass LEDEmbedded = 
{
    "LEDEmbedded",
    NULL,
    instantiate_LEDEmbedded,
    0,
    NULL,
    2,
    portroles_border,
    1,
    portroles_internal
};

